#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_NLS
#include <libintl.h>
#endif

#define         _ISOC9X_SOURCE  1
#define         _ISOC99_SOURCE  1
#define         __USE_ISOC99    1
#define         __USE_ISOC9X    1

#include <math.h>
#include "utils.h"
#include "ladspa.h"


#define FOO_CHOP_LIVER_CHOP_WINDOW     0
#define FOO_CHOP_LIVER_DRY_GAIN        1
#define FOO_CHOP_LIVER_WET_GAIN        2
#define FOO_CHOP_LIVER_INPUT_BUFFER    3
#define FOO_CHOP_LIVER_OUTPUT_BUFFER   4
#define FOO_CHOP_LIVER_LATENCY         5

static LADSPA_Descriptor *foo_chop_liverDescriptor = NULL;

typedef struct {
	LADSPA_Data *chop_window;
	LADSPA_Data *dry_gain;
	LADSPA_Data *wet_gain;
	LADSPA_Data *input_buffer;
	LADSPA_Data *output_buffer;
	LADSPA_Data *latency;
	LADSPA_Data *window;
	LADSPA_Data run_adding_gain;
} Foo_chop_liver;

const LADSPA_Descriptor *ladspa_descriptor(unsigned long index) {

	switch (index) {
	case 0:
		return foo_chop_liverDescriptor;
	default:
		return NULL;
	}
}

static void activateFoo_chop_liver(LADSPA_Handle instance) {
	Foo_chop_liver *plugin_data = (Foo_chop_liver *)instance;
	LADSPA_Data *window = plugin_data->window;
#line 43 "foo_chop.xml"
	*(plugin_data->latency) = 0;
	plugin_data->window = window;

}

static void cleanupFoo_chop_liver(LADSPA_Handle instance) {
#line 47 "foo_chop.xml"
	Foo_chop_liver *plugin_data = (Foo_chop_liver *)instance;
	free(plugin_data->window);
	free(instance);
}

static void connectPortFoo_chop_liver(
 LADSPA_Handle instance,
 unsigned long port,
 LADSPA_Data *data) {
	Foo_chop_liver *plugin;

	plugin = (Foo_chop_liver *)instance;
	switch (port) {
	case FOO_CHOP_LIVER_CHOP_WINDOW:
		plugin->chop_window = data;
		break;
	case FOO_CHOP_LIVER_DRY_GAIN:
		plugin->dry_gain = data;
		break;
	case FOO_CHOP_LIVER_WET_GAIN:
		plugin->wet_gain = data;
		break;
	case FOO_CHOP_LIVER_INPUT_BUFFER:
		plugin->input_buffer = data;
		break;
	case FOO_CHOP_LIVER_OUTPUT_BUFFER:
		plugin->output_buffer = data;
		break;
	case FOO_CHOP_LIVER_LATENCY:
		plugin->latency = data;
		break;
	}
}

static LADSPA_Handle instantiateFoo_chop_liver(
 const LADSPA_Descriptor *descriptor,
 unsigned long s_rate) {
	Foo_chop_liver *plugin_data = (Foo_chop_liver *)malloc(sizeof(Foo_chop_liver));
	LADSPA_Data *window = NULL;

#line 36 "foo_chop.xml"
	window = (LADSPA_Data *) malloc(32*sizeof(LADSPA_Data)); 
	// much match the value in port definitions

	// Nothing to do beautiful

	plugin_data->window = window;

	return (LADSPA_Handle)plugin_data;
}

#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b = v)
#define RUN_ADDING    0
#define RUN_REPLACING 1

static void runFoo_chop_liver(LADSPA_Handle instance, unsigned long sample_count) {
	Foo_chop_liver *plugin_data = (Foo_chop_liver *)instance;

	/* Chop window (float value) */
	const LADSPA_Data chop_window = *(plugin_data->chop_window);

	/* Dry signal (float value) */
	const LADSPA_Data dry_gain = *(plugin_data->dry_gain);

	/* Wet signal (float value) */
	const LADSPA_Data wet_gain = *(plugin_data->wet_gain);

	/* Input (array of floats of length sample_count) */
	const LADSPA_Data * const input_buffer = plugin_data->input_buffer;

	/* Output (array of floats of length sample_count) */
	LADSPA_Data * const output_buffer = plugin_data->output_buffer;
	LADSPA_Data * window = plugin_data->window;

#line 51 "foo_chop.xml"
	int i;
	int window_size = (int)chop_window;
	//printf("window_size = %d\n",window_size);

	LADSPA_Data * input = input_buffer;
	LADSPA_Data * output = output_buffer;

	if (window_size < 2) window_size = 2;
	if (window_size > 32) window_size = 32;

	while (sample_count > 0) {
	        //printf("samples left %d, window %d\n",sample_count, window_size);
	        
	        for (i = 0; i < window_size; ++i) {
	                window[i] = input[window_size - i - 1];
	        }

	        for (i = 0; i < window_size; ++i) {
	                buffer_write(output[i], 
	                                input [i] * dry_gain +
	                                window[i] * wet_gain);
	        }

	        output += window_size;
	        input  += window_size;

	        sample_count -= window_size;

	        // last window will be the size of the samples left over
	        if (window_size > sample_count)
	                window_size = sample_count; 
	}
}
#undef buffer_write
#undef RUN_ADDING
#undef RUN_REPLACING

#define buffer_write(b, v) (b += (v) * run_adding_gain)
#define RUN_ADDING    1
#define RUN_REPLACING 0

static void setRunAddingGainFoo_chop_liver(LADSPA_Handle instance, LADSPA_Data gain) {
	((Foo_chop_liver *)instance)->run_adding_gain = gain;
}

static void runAddingFoo_chop_liver(LADSPA_Handle instance, unsigned long sample_count) {
	Foo_chop_liver *plugin_data = (Foo_chop_liver *)instance;
	LADSPA_Data run_adding_gain = plugin_data->run_adding_gain;

	/* Chop window (float value) */
	const LADSPA_Data chop_window = *(plugin_data->chop_window);

	/* Dry signal (float value) */
	const LADSPA_Data dry_gain = *(plugin_data->dry_gain);

	/* Wet signal (float value) */
	const LADSPA_Data wet_gain = *(plugin_data->wet_gain);

	/* Input (array of floats of length sample_count) */
	const LADSPA_Data * const input_buffer = plugin_data->input_buffer;

	/* Output (array of floats of length sample_count) */
	LADSPA_Data * const output_buffer = plugin_data->output_buffer;
	LADSPA_Data * window = plugin_data->window;

#line 51 "foo_chop.xml"
	int i;
	int window_size = (int)chop_window;
	//printf("window_size = %d\n",window_size);

	LADSPA_Data * input = input_buffer;
	LADSPA_Data * output = output_buffer;

	if (window_size < 2) window_size = 2;
	if (window_size > 32) window_size = 32;

	while (sample_count > 0) {
	        //printf("samples left %d, window %d\n",sample_count, window_size);
	        
	        for (i = 0; i < window_size; ++i) {
	                window[i] = input[window_size - i - 1];
	        }

	        for (i = 0; i < window_size; ++i) {
	                buffer_write(output[i], 
	                                input [i] * dry_gain +
	                                window[i] * wet_gain);
	        }

	        output += window_size;
	        input  += window_size;

	        sample_count -= window_size;

	        // last window will be the size of the samples left over
	        if (window_size > sample_count)
	                window_size = sample_count; 
	}
}

#ifndef FOO_TESTER
void _init() {
#else
void fake_init() {
#endif
	char **port_names;
	LADSPA_PortDescriptor *port_descriptors;
	LADSPA_PortRangeHint *port_range_hints;

#ifdef ENABLE_NLS
#define D_(s) dgettext(PACKAGE, s)
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
#else
#define D_(s) (s)
#endif


	foo_chop_liverDescriptor =
	 (LADSPA_Descriptor *)malloc(sizeof(LADSPA_Descriptor));

	if (foo_chop_liverDescriptor) {
		foo_chop_liverDescriptor->UniqueID = 3188;
		foo_chop_liverDescriptor->Label = "foo_chop_liver";
		foo_chop_liverDescriptor->Properties =
		 LADSPA_PROPERTY_HARD_RT_CAPABLE;
		foo_chop_liverDescriptor->Name =
		 D_("Foo Chop Liver");
		foo_chop_liverDescriptor->Maker =
		 "Sampo Savolainen <v2@iki.fi>";
		foo_chop_liverDescriptor->Copyright =
		 "GPL";
		foo_chop_liverDescriptor->PortCount = 6;

		port_descriptors = (LADSPA_PortDescriptor *)calloc(6,
		 sizeof(LADSPA_PortDescriptor));
		foo_chop_liverDescriptor->PortDescriptors =
		 (const LADSPA_PortDescriptor *)port_descriptors;

		port_range_hints = (LADSPA_PortRangeHint *)calloc(6,
		 sizeof(LADSPA_PortRangeHint));
		foo_chop_liverDescriptor->PortRangeHints =
		 (const LADSPA_PortRangeHint *)port_range_hints;

		port_names = (char **)calloc(6, sizeof(char*));
		foo_chop_liverDescriptor->PortNames =
		 (const char **)port_names;

		/* Parameters for Chop window */
		port_descriptors[FOO_CHOP_LIVER_CHOP_WINDOW] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
		port_names[FOO_CHOP_LIVER_CHOP_WINDOW] =
		 D_("Chop window");
		port_range_hints[FOO_CHOP_LIVER_CHOP_WINDOW].HintDescriptor =
		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_INTEGER | LADSPA_HINT_DEFAULT_MINIMUM;
		port_range_hints[FOO_CHOP_LIVER_CHOP_WINDOW].LowerBound = 2;
		port_range_hints[FOO_CHOP_LIVER_CHOP_WINDOW].UpperBound = 32;

		/* Parameters for Dry signal */
		port_descriptors[FOO_CHOP_LIVER_DRY_GAIN] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
		port_names[FOO_CHOP_LIVER_DRY_GAIN] =
		 D_("Dry signal");
		port_range_hints[FOO_CHOP_LIVER_DRY_GAIN].HintDescriptor =
		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0;
		port_range_hints[FOO_CHOP_LIVER_DRY_GAIN].LowerBound = 0.0;
		port_range_hints[FOO_CHOP_LIVER_DRY_GAIN].UpperBound = +1.0;

		/* Parameters for Wet signal */
		port_descriptors[FOO_CHOP_LIVER_WET_GAIN] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL;
		port_names[FOO_CHOP_LIVER_WET_GAIN] =
		 D_("Wet signal");
		port_range_hints[FOO_CHOP_LIVER_WET_GAIN].HintDescriptor =
		 LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_1;
		port_range_hints[FOO_CHOP_LIVER_WET_GAIN].LowerBound = 0.0;
		port_range_hints[FOO_CHOP_LIVER_WET_GAIN].UpperBound = +1.0;

		/* Parameters for Input */
		port_descriptors[FOO_CHOP_LIVER_INPUT_BUFFER] =
		 LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
		port_names[FOO_CHOP_LIVER_INPUT_BUFFER] =
		 D_("Input");
		port_range_hints[FOO_CHOP_LIVER_INPUT_BUFFER].HintDescriptor = 0;

		/* Parameters for Output */
		port_descriptors[FOO_CHOP_LIVER_OUTPUT_BUFFER] =
		 LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
		port_names[FOO_CHOP_LIVER_OUTPUT_BUFFER] =
		 D_("Output");
		port_range_hints[FOO_CHOP_LIVER_OUTPUT_BUFFER].HintDescriptor = 0;

		/* Parameters for latency */
		port_descriptors[FOO_CHOP_LIVER_LATENCY] =
		 LADSPA_PORT_OUTPUT | LADSPA_PORT_CONTROL;
		port_names[FOO_CHOP_LIVER_LATENCY] =
		 D_("latency");
		port_range_hints[FOO_CHOP_LIVER_LATENCY].HintDescriptor = 0;

		foo_chop_liverDescriptor->activate = activateFoo_chop_liver;
		foo_chop_liverDescriptor->cleanup = cleanupFoo_chop_liver;
		foo_chop_liverDescriptor->connect_port = connectPortFoo_chop_liver;
		foo_chop_liverDescriptor->deactivate = NULL;
		foo_chop_liverDescriptor->instantiate = instantiateFoo_chop_liver;
		foo_chop_liverDescriptor->run = runFoo_chop_liver;
		foo_chop_liverDescriptor->run_adding = runAddingFoo_chop_liver;
		foo_chop_liverDescriptor->set_run_adding_gain = setRunAddingGainFoo_chop_liver;
	}
}

#ifndef FOO_TESTER
void _fini() {
#else
void fake_fini() {
#endif
	if (foo_chop_liverDescriptor) {
		free((LADSPA_PortDescriptor *)foo_chop_liverDescriptor->PortDescriptors);
		free((char **)foo_chop_liverDescriptor->PortNames);
		free((LADSPA_PortRangeHint *)foo_chop_liverDescriptor->PortRangeHints);
		free(foo_chop_liverDescriptor);
	}

}
